/*******************************************************************************
 *	@file			DLGraph.cpp
 *	@brief			DirectX Library
 *
 *	@date			21-Apr-2010(Wed)
 ******************************************************************************/

// vvZbT---------------------------------------------------------------
#include "DLGraph.h"

/*******************************************************************************
 *	@namespace	DL
 ******************************************************************************/
namespace	DL
{

// O[oϐ---------------------------------------------------------
// o[֐|C^ifޓǂݍ݁j
int ( CDLGraph::*CDLGraph::StepLoad[ enumMaterial_GraphEnd - enumMaterial_GraphStart ] )( const SDLMaterial& SDL, const int& nNum ) =
{
	&CDLGraph::LoadBmp, &CDLGraph::LoadFont,
};
// o[֐|C^ifމj
int ( CDLGraph::*CDLGraph::StepRemove[ enumMaterial_GraphEnd - enumMaterial_GraphStart ] )( const int& nNum ) =
{
	&CDLGraph::RemoveBmp, &CDLGraph::RemoveFont,
};
// o[֐|C^iIuWFNgǉj
int ( CDLGraph::*CDLGraph::StepAdd[ enumObject_Max ] )( const SDLGraph& SDL, const int& nNum ) =
{
	NULL,
	&CDLGraph::AddRect, &CDLGraph::AddPlane,
	&CDLGraph::AddText,
};
// o[֐|C^iIuWFNg폜j
int ( CDLGraph::*CDLGraph::StepDel[ enumObject_Max ] )( const int& nNum ) =
{
	NULL,
	&CDLGraph::DelRect, &CDLGraph::DelPlane,
	&CDLGraph::DelText,
};
// o[֐|C^iIuWFNg`j
int ( CDLGraph::*CDLGraph::StepDraw[ enumObject_Max ] )( D3DOBJECT *pObj ) =
{
	NULL,
	&CDLGraph::DrawPrimitiveUP, &CDLGraph::DrawPrimitiveUP,
	&CDLGraph::DrawText,
};

// ֐vg^Cv錾---------------------------------------------------

// -----------------------------------------------------------------------------
/*******************************************************************************
 *	@fn				BOOL CDLGraph::SetViewPort( const SDLParam& SDL )
 *	@param[in]		SDL							DL\
 *	@retval			int							TRUE=OK/FALSE=NG
 *	@brief			r[|[g̐ݒ
 ******************************************************************************/
BOOL CDLGraph::SetViewPort( const SDLParam& SDL )
{
	HRESULT hr = D3D_OK;
	// http://msdn.microsoft.com/ja-jp/library/cc323943.aspx
	m_D3DVP.X		= 0;
	m_D3DVP.Y		= 0;
	m_D3DVP.Width	= SDL.m_nWidth;
	m_D3DVP.Height	= SDL.m_nHeight;
	m_D3DVP.MinZ	= 0.0f;
	m_D3DVP.MaxZ	= 1.0f;
	hr = m_lpD3DDev->SetViewport( &m_D3DVP );
	if( FAILED( hr ) )
	{
		DXTRACE_ERR( "SetViewport()", hr );
		return FALSE;
	}

	return TRUE;
}

/*******************************************************************************
 *	@fn				BOOL CDLGraph::DeviceRestore( const HRESULT& hRes )
 *	@retval			none						Ȃ
 *	@brief			foCX̕
 ******************************************************************************/
BOOL CDLGraph::DeviceRestore( const HRESULT& hRes )
{
	// foCX̊mF
	if( SUCCEEDED( hRes ) ) return TRUE;

	HRESULT hr;
	// foCX̏
	hr = m_lpD3DDev->TestCooperativeLevel();
	if( FAILED( hr ) )
	{
		// foCX͂܂Ă
		if( hr == D3DERR_DEVICELOST )
			return TRUE;
		// \ʃG[
		if( hr != D3DERR_DEVICENOTRESET )
			return FALSE;

		// \[X

		// Ă݂
		hr = m_lpD3DDev->Reset( &m_D3DPP );
		if( FAILED( hr ) )
		{
			// foCX͂܂Ă
			if( hr == D3DERR_DEVICELOST )
				return TRUE;
			return FALSE;
		}

		// \[XĎ擾
	}

	return TRUE;
}

/*******************************************************************************
 *	@fn				void CDLGraph::SetState( const SDLGraph& SDL, D3DOBJECT* pObj )
 *	@param[in]		SDL							Graphics\
 *	@param[in]		pObj						IuWFNg\
 *	@retval			int							TRUE=OK/FALSE=NG
 *	@brief			Xe[g̐ݒ
 ******************************************************************************/
void CDLGraph::SetState( const SDLGraph& SDL, D3DOBJECT* pObj )
{
	HRESULT hr = D3D_OK;
	// L^Jn
	hr = m_lpD3DDev->BeginStateBlock();

	// http://marupeke296.com/DXG_TextureStage.html
	// http://www.tom.sfc.keio.ac.jp/~fjedi/wiki/index.php?%A5%C6%A5%AF%A5%B9%A5%C1%A5%E3
	switch( SDL.m_eState )
	{
	case enumState_Null:					// Ȃ
		// eNX`EXe[WEXe[g
		hr = m_lpD3DDev->SetTextureStageState( 0, D3DTSS_COLOROP, D3DTOP_SELECTARG1 );
		// _OEXe[g̐ݒ
		hr = m_lpD3DDev->SetRenderState( D3DRS_ZENABLE, FALSE );				//!< yobt@O
		hr = m_lpD3DDev->SetRenderState( D3DRS_LIGHTING, FALSE );				//!< CeBO
		hr = m_lpD3DDev->SetRenderState( D3DRS_ALPHABLENDENABLE, FALSE );		//!< At@uh
		break;
	case enumState_Translucent:				// 
		// eNX`EXe[WEXe[g
		hr = m_lpD3DDev->SetTextureStageState( 0, D3DTSS_ALPHAOP, D3DTOP_MODULATE );
		// _OEXe[g̐ݒ
		hr = m_lpD3DDev->SetRenderState( D3DRS_ALPHABLENDENABLE, TRUE );
		hr = m_lpD3DDev->SetRenderState( D3DRS_SRCBLEND, D3DBLEND_SRCALPHA );
		hr = m_lpD3DDev->SetRenderState( D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA );
		break;
	case enumState_Add:						//!< Z
		// eNX`EXe[WEXe[g
		hr = m_lpD3DDev->SetTextureStageState( 0, D3DTSS_ALPHAOP, D3DTOP_ADD );
		// _OEXe[g̐ݒ
		hr = m_lpD3DDev->SetRenderState( D3DRS_ALPHABLENDENABLE, TRUE );
		hr = m_lpD3DDev->SetRenderState( D3DRS_SRCBLEND, D3DBLEND_ONE );
		hr = m_lpD3DDev->SetRenderState( D3DRS_DESTBLEND, D3DBLEND_ONE );
		break;
	case enumState_Sub:						//!< Z
		// eNX`EXe[WEXe[g
		hr = m_lpD3DDev->SetTextureStageState( 0, D3DTSS_ALPHAOP, D3DTOP_SUBTRACT );
		// _OEXe[g̐ݒ
		hr = m_lpD3DDev->SetRenderState( D3DRS_ALPHABLENDENABLE, TRUE );
		hr = m_lpD3DDev->SetRenderState( D3DRS_SRCBLEND, D3DBLEND_ZERO );
		hr = m_lpD3DDev->SetRenderState( D3DRS_DESTBLEND, D3DBLEND_INVSRCCOLOR );
		break;
	}

	// L^I
	hr = m_lpD3DDev->EndStateBlock( &pObj->m_pD3DState );
}

/*******************************************************************************
 *	@fn				int CDLGraph::LoadBmp( const SDLMaterial& SDL, const int& nNum )
 *	@param[in]		SDL							fޏ\
 *	@param[in]		nNum						fޔԍ
 *	@retval			int							TRUE=OK/FALSE=NG
 *	@brief			摜ǂݍ
 ******************************************************************************/
int CDLGraph::LoadBmp( const SDLMaterial& SDL, const int& nNum )
{
	HRESULT hr = D3D_OK;
	D3DTEXTURE* pTex = &m_D3DTex[ nNum ];
	pTex->m_szName = SDL.m_szName;
	pTex->m_dwColor = SDL.m_dwColor;

	// eNX`̐ݒ
	// 摜̓ǂݍ
	hr = D3DXCreateTextureFromFileEx(
			m_lpD3DDev,
			pTex->m_szName,
			D3DX_DEFAULT_NONPOW2,
			D3DX_DEFAULT_NONPOW2,
			D3DFMT_FROM_FILE,
			0,
			D3DFMT_UNKNOWN,
			D3DPOOL_MANAGED,
			D3DX_DEFAULT,
			D3DX_DEFAULT,
			pTex->m_dwColor,
			NULL,
			NULL,
			&pTex->m_lpD3DTex );
	if( FAILED( hr ) )
	{
		DXTRACE_MSG( "D3DXCreateTextureFromFileEx()" );
		return FALSE;
	}
	// \[X擾
	hr = pTex->m_lpD3DTex->GetLevelDesc( 0, &pTex->m_D3DDesc );
	if( FAILED( hr ) )
	{
		DXTRACE_MSG( "GetLevelDesc()" );
		return FALSE;
	}

	return TRUE;
}

/*******************************************************************************
 *	@fn				int CDLGraph::LoadBmp( const SDLMaterial& SDL, const int& nNum )
 *	@param[in]		SDL							fޏ\
 *	@param[in]		nNum						fޔԍ
 *	@retval			int							TRUE=OK/FALSE=NG
 *	@brief			tHg쐬
 ******************************************************************************/
int CDLGraph::LoadFont( const SDLMaterial& SDL, const int& nNum )
{
	HRESULT hr = D3D_OK;
	D3DFONT* pFont = &m_D3DFont[ nNum ];

	// tHg̐ݒ
	hr = D3DXCreateFont(
			m_lpD3DDev,
			SDL.m_nHeight,
			0,
			FW_NORMAL,
			0,
			FALSE,
			SHIFTJIS_CHARSET,
			OUT_DEFAULT_PRECIS,
			DEFAULT_QUALITY,
			DEFAULT_PITCH | FF_ROMAN,
			SDL.m_szName,
			&pFont->m_lpD3DFont );
	if( FAILED( hr ) )
	{
		DXTRACE_MSG( "D3DXCreateFont()" );
		return FALSE;
	}
	return TRUE;
}

/*******************************************************************************
 *	@fn				int CDLGraph::RemoveBmp( const int& nNum )
 *	@param[in]		nNum						fޔԍ
 *	@retval			int							TRUE=OK/FALSE=NG
 *	@brief			摜
 ******************************************************************************/
int CDLGraph::RemoveBmp( const int& nNum )
{
	D3DTEXTURE* pTex = &m_D3DTex[ nNum ];

	// 
	pTex->m_szName = "";
	_RELEASE( pTex->m_lpD3DTex );

	// 
	D3DTEXTURE n;
	*pTex = n;

	return TRUE;
}

/*******************************************************************************
 *	@fn				int CDLGraph::RemoveFont( const int& nNum )
 *	@param[in]		nNum						fޔԍ
 *	@retval			int							TRUE=OK/FALSE=NG
 *	@brief			tHg
 ******************************************************************************/
int CDLGraph::RemoveFont( const int& nNum )
{
	D3DFONT* pFont = &m_D3DFont[ nNum ];

	// 
	_RELEASE( pFont->m_lpD3DFont );

	// 
	D3DFONT n;
	*pFont = n;

	return TRUE;
}

/*******************************************************************************
 *	@fn				int CDLGraph::AddRect( const SDLGraph& SDL, const int& nNum )
 *	@param[in]		SDL							Graphics\
 *	@param[in]		nNum						IuWFNgԍ
 *	@retval			int							TRUE=OK/FALSE=NG
 *	@brief			`ǉ
 ******************************************************************************/
int CDLGraph::AddRect( const SDLGraph& SDL, const int& nNum )
{
	D3DOBJECT* pObj = &m_D3DObj[ nNum ];
	RECT* pRect = static_cast<RECT*>( SDL.m_pData0 );
	int* pColor = static_cast<int*>( SDL.m_pData1 );

	// {o^
	pObj->m_bUsed = TRUE;
	pObj->m_eObj = SDL.m_eObj;
	pObj->m_nPrio = SDL.m_nPrio;
	// fޔԍo^
	pObj->m_nMatNum = SDL.m_nMatNum;
	// `o^
	pObj->m_eDraw = SDL.m_eDraw;
	SetState( SDL, pObj );
	// f[^o^
	pObj->m_pObj = static_cast<D3DDATA*>( ::malloc( sizeof( D3DDATA ) ) );
	D3DDATA* pData = static_cast<D3DDATA*>( pObj->m_pObj );

	pData->m_nDataMax = 4;
	pData->m_pData = static_cast<D3DVERTEX*>( ::malloc( pData->m_nDataMax * sizeof( D3DVERTEX ) ) );

	D3DVERTEX v[ 4 ] =
	{
		{ static_cast<float>( (*pRect).left  ), static_cast<float>( (*pRect).top    ), 0.0f, 1.0f, *pColor, 0.0f, 0.0f },			// 
		{ static_cast<float>( (*pRect).right ), static_cast<float>( (*pRect).top    ), 0.0f, 1.0f, *pColor, 1.0f, 0.0f },			// E
		{ static_cast<float>( (*pRect).right ), static_cast<float>( (*pRect).bottom ), 0.0f, 1.0f, *pColor, 1.0f, 1.0f },			// E
		{ static_cast<float>( (*pRect).left  ), static_cast<float>( (*pRect).bottom ), 0.0f, 1.0f, *pColor, 0.0f, 1.0f },			// 
	};

	for( int i = 0; i < pData->m_nDataMax; i++ )
		pData->m_pData[ i ] = v[ i ];

	return TRUE;
}

/*******************************************************************************
 *	@fn				int CDLGraph::AddPlane( const SDLGraph& SDL, const int& nNum )
 *	@param[in]		SDL							Graphics\
 *	@param[in]		nNum						IuWFNgԍ
 *	@retval			int							TRUE=OK/FALSE=NG
 *	@brief			ʒǉ
 ******************************************************************************/
int CDLGraph::AddPlane( const SDLGraph& SDL, const int& nNum )
{
	D3DOBJECT* pObj = &m_D3DObj[ nNum ];
	int* pPlane = static_cast<int*>( SDL.m_pData0 );
	SDLPlane* pSPln = static_cast<SDLPlane*>( SDL.m_pData1 );

	// {o^
	pObj->m_bUsed = TRUE;
	pObj->m_eObj = SDL.m_eObj;
	pObj->m_nPrio = SDL.m_nPrio;
	// fޔԍo^
	pObj->m_nMatNum = SDL.m_nMatNum;
	// `o^
	pObj->m_eDraw = SDL.m_eDraw;
	SetState( SDL, pObj );
	// f[^o^
	pObj->m_pObj = static_cast<D3DDATA*>( ::malloc( sizeof( D3DDATA ) ) );
	D3DDATA* pData = static_cast<D3DDATA*>( pObj->m_pObj );

	pData->m_nDataMax = *pPlane;
	pData->m_pData = static_cast<D3DVERTEX*>( ::malloc( pData->m_nDataMax * sizeof( D3DVERTEX ) ) );

	for( int i = 0; i < pData->m_nDataMax; i++ )
	{
		D3DVERTEX v;
		v.x = pSPln[ i ].m_fX; v.y = pSPln[ i ].m_fY; v.z = pSPln[ i ].m_fZ;
		v.rhw = 1.0f; v.color = pSPln[ i ].m_nC;

		pData->m_pData[ i ] = v;
	}

	return TRUE;
}

/*******************************************************************************
 *	@fn				int CDLGraph::AddText( const SDLGraph& SDL, const int& nNum )
 *	@param[in]		SDL							Graphics\
 *	@param[in]		nNum						IuWFNgԍ
 *	@retval			int							TRUE=OK/FALSE=NG
 *	@brief			eLXgǉ
 ******************************************************************************/
int CDLGraph::AddText( const SDLGraph& SDL, const int& nNum )
{
	D3DOBJECT* pObj = &m_D3DObj[ nNum ];
	RECT* pRect = static_cast<RECT*>( SDL.m_pData0 );
	int* pColor = static_cast<int*>( SDL.m_pData1 );
	LPSTR* lpStr = static_cast<LPSTR*>( SDL.m_pData2 );

	// gp
	if( SDL.m_nMatNum == -1 ) return FALSE;

	// {o^
	pObj->m_bUsed = TRUE;
	pObj->m_eObj = SDL.m_eObj;
	pObj->m_nPrio = SDL.m_nPrio;
	// fޔԍo^
	pObj->m_nMatNum = SDL.m_nMatNum;
	// `o^
	pObj->m_eDraw = SDL.m_eDraw;

	// f[^o^
	pObj->m_pObj = static_cast<D3DTEXT*>( ::malloc( sizeof( D3DTEXT ) ) );
	D3DTEXT* pText = static_cast<D3DTEXT*>( pObj->m_pObj );

	pText->m_reRect = *pRect;
	pText->m_dwColor = D3DCOLOR_XRGB( ( *pColor & 0x00FF0000 ) >> 16,
									  ( *pColor & 0x0000FF00 ) >>  8,
									  ( *pColor & 0x000000FF ) >>  0 );
	pText->m_szStr = *lpStr;


	return TRUE;
}

/*******************************************************************************
 *	@fn				int CDLGraph::DelNull( const int& nNum )
 *	@param[in]		nNum						IuWFNgԍ
 *	@retval			int							TRUE=OK/FALSE=NG
 *	@brief			`폜
 ******************************************************************************/
int CDLGraph::DelRect( const int& nNum )
{
	D3DOBJECT* pObj = &m_D3DObj[ nNum ];

	// 
	_RELEASE( pObj->m_pD3DState );
	D3DDATA* pData = static_cast<D3DDATA*>( pObj->m_pObj );
	if( pData != NULL )
		_MEMCHECK( pData->m_pData );
	_MEMCHECK( pObj->m_pObj );

	// 
	D3DOBJECT n;
	*pObj = n;

	return TRUE;
}

/*******************************************************************************
 *	@fn				int CDLGraph::DelPlane( const int& nNum )
 *	@param[in]		nNum						IuWFNgԍ
 *	@retval			int							TRUE=OK/FALSE=NG
 *	@brief			ʍ폜
 ******************************************************************************/
int CDLGraph::DelPlane( const int& nNum )
{
	D3DOBJECT* pObj = &m_D3DObj[ nNum ];

	// 
	_RELEASE( pObj->m_pD3DState );
	D3DDATA* pData = static_cast<D3DDATA*>( pObj->m_pObj );
	if( pData != NULL )
		_MEMCHECK( pData->m_pData );
	_MEMCHECK( pObj->m_pObj );

	// 
	D3DOBJECT n;
	*pObj = n;

	return TRUE;
}

/*******************************************************************************
 *	@fn				int CDLGraph::DelText( const int& nNum )
 *	@param[in]		nNum						IuWFNgԍ
 *	@retval			int							TRUE=OK/FALSE=NG
 *	@brief			eLXg폜
 ******************************************************************************/
int CDLGraph::DelText( const int& nNum )
{
	D3DOBJECT* pObj = &m_D3DObj[ nNum ];

	// 
	D3DTEXT* pText = static_cast<D3DTEXT*>( pObj->m_pObj );
	if( pText != NULL )
		pText->m_szStr = "";
	_MEMCHECK( pObj->m_pObj );

	// 
	D3DOBJECT n;
	*pObj = n;

	return TRUE;
}

/*******************************************************************************
 *	@fn				int CDLGraph::DrawPrimitiveUP( D3DOBJECT *pObj )
 *	@param[in]		pObj						IuWFNg\
 *	@retval			int							TRUE=OK/FALSE=NG
 *	@brief			v~eBu`
 ******************************************************************************/
int CDLGraph::DrawPrimitiveUP( D3DOBJECT *pObj )
{
	HRESULT hr = D3D_OK;
	// Xe[g̐ݒ
	hr = pObj->m_pD3DState->Apply();
	// eNX`̐ݒ
	hr = m_lpD3DDev->SetTexture( 0, ( pObj->m_nMatNum >= 0 ) ? m_D3DTex[ pObj->m_nMatNum ].m_lpD3DTex : NULL );
	// _Xg[̐錾ݒ
	hr = m_lpD3DDev->SetFVF( D3DFVF_XYZRHW | D3DFVF_DIFFUSE | D3DFVF_TEX1 );
	D3DDATA* pData = static_cast<D3DDATA*>( pObj->m_pObj );
	// v~eBu`
	hr = m_lpD3DDev->DrawPrimitiveUP( D3DPT_TRIANGLEFAN,
									  pData->m_nDataMax - 2,
									  pData->m_pData,
									  sizeof( D3DVERTEX ) );

	return TRUE;
}

/*******************************************************************************
 *	@fn				int CDLGraph::DrawText( D3DOBJECT *pObj )
 *	@param[in]		pObj						IuWFNg\
 *	@retval			int							TRUE=OK/FALSE=NG
 *	@brief			eLXg`
 ******************************************************************************/
int CDLGraph::DrawText( D3DOBJECT *pObj )
{
	HRESULT hr = D3D_OK;
	D3DFONT* pFont = &m_D3DFont[ pObj->m_nMatNum ];
	D3DTEXT* pText = static_cast<D3DTEXT*>( pObj->m_pObj );
	// eLXg̕`
	hr = pFont->m_lpD3DFont->DrawText( NULL,
									   pText->m_szStr,
									   -1,
									   &pText->m_reRect,
									   DT_LEFT | DT_EXPANDTABS,
									   pText->m_dwColor );

	return TRUE;
}

/*******************************************************************************
 *	@fn				CDLGraph::CDLGraph( void )
 *	@retval			none						Ȃ
 *	@brief			RXgN^
 ******************************************************************************/
CDLGraph::CDLGraph( void )
	: m_lpD3D( NULL )
	, m_lpD3DDev( NULL )

	, m_hWnd()
	, m_D3DPP()
	, m_D3DDM()
	, m_D3DVP()

	, m_nOTMax( 0 )
{
	// `e[u
	for( int i = 0; i < OBJECT_MAX; i++ )
		m_nOT[ i ] = -1;
}

/*******************************************************************************
 *	@fn				CDLGraph::~CDLGraph( void )
 *	@retval			none						Ȃ
 *	@brief			fXgN^
 ******************************************************************************/
CDLGraph::~CDLGraph( void )
{
}

/*******************************************************************************
 *	@fn				int CDLGraph::Init( const SDLParam& SDL )
 *	@param[in]		SDL							DL\
 *	@retval			int							TRUE=OK/FALSE=NG
 *	@brief			
 ******************************************************************************/
int CDLGraph::Init( const SDLParam& SDL )
{
	m_hWnd = SDL.m_hWnd;
	HRESULT hr = D3D_OK;

	// IDirect3D9 C^[tFCX̎擾
	// http://msdn.microsoft.com/ja-jp/library/cc324303.aspx
	m_lpD3D = ::Direct3DCreate9( D3D_SDK_VERSION );
	if( m_lpD3D == NULL )
	{
		DXTRACE_MSG( "Direct3DCreate9()" );
		return FALSE;
	}

	// foCX\̂̏
	::ZeroMemory( &m_D3DPP, sizeof( D3DPRESENT_PARAMETERS ) );

	// ݂̉ʃ[h擾܂
	// http://msdn.microsoft.com/ja-jp/library/cc324298.aspx
	hr = m_lpD3D->GetAdapterDisplayMode( D3DADAPTER_DEFAULT, &m_D3DDM );
	if( FAILED( hr ) )
	{
		DXTRACE_ERR( "GetAdapterDisplayMode()", hr );
		return FALSE;
	}

	// foCX\̂̐ݒ
	// http://msdn.microsoft.com/ja-jp/library/cc323931.aspx
	m_D3DPP.Windowed	= SDL.m_bMode;
	m_D3DPP.SwapEffect	= D3DSWAPEFFECT_DISCARD;
	m_D3DPP.BackBufferFormat	= m_D3DDM.Format;
	m_D3DPP.BackBufferWidth		= SDL.m_nWidth;
	m_D3DPP.BackBufferHeight	= SDL.m_nHeight;
	m_D3DPP.BackBufferCount		= 1;

	// foCX̊mF
	hr = m_lpD3D->CheckDeviceType( D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, m_D3DPP.BackBufferFormat, m_D3DPP.BackBufferFormat, FALSE );
	if( FAILED( hr ) )
		DXTRACE_ERR( "CheckDeviceType()", hr );

	// IDirect3Device9 C^[tFCX̎擾
	// http://msdn.microsoft.com/ja-jp/library/cc324295.aspx
	// HAL + HARDWARE
	hr = m_lpD3D->CreateDevice( D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, m_hWnd, D3DCREATE_HARDWARE_VERTEXPROCESSING | D3DCREATE_MULTITHREADED, &m_D3DPP, &m_lpD3DDev );
	if( FAILED( hr ) )
	{
		DXTRACE_ERR( "CreateDevice()", hr );
		// HAL + SOFTWARE
		hr = m_lpD3D->CreateDevice( D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, m_hWnd, D3DCREATE_SOFTWARE_VERTEXPROCESSING | D3DCREATE_MULTITHREADED, &m_D3DPP, &m_lpD3DDev );
		if( FAILED( hr ) )
		{
			// REF + SOFTWARE ͎gpȂ
			DXTRACE_ERR( "CreateDevice()", hr );
			return FALSE;
		}
	}

	// r[|[g̐ݒ
	if( !SetViewPort( SDL ) )
		return FALSE;

	return TRUE;
}

/*******************************************************************************
 *	@fn				int CDLGraph::End( void )
 *	@retval			int							TRUE=OK/FALSE=NG
 *	@brief			I
 ******************************************************************************/
int CDLGraph::End( void )
{
	// IDirect3Device9 C^[tFCX̉
	_RELEASE( m_lpD3DDev );
	// IDirect3D9 C^[tFCX̉
	_RELEASE( m_lpD3D );

	return TRUE;
}

/*******************************************************************************
 *	@fn				int CDLGraph::Sort( void )
 *	@retval			int							TRUE=OK/FALSE=NG
 *	@brief			\[g
 ******************************************************************************/
int CDLGraph::Sort( void )
{
	m_nOTMax = 0;
	// IuWFNgo^
	for( int i = 0; i < OBJECT_MAX; i++ )
	{
		D3DOBJECT *pObj = &m_D3DObj[ i ];
		// gpĂȂꍇ͓o^Ȃ
		if( !pObj->m_bUsed ) continue;
		// gpIuWFNge[uɓo^
		m_nOT[ m_nOTMax++ ] = i;
	}
	// ou\[g
	if( m_nOTMax > 0 )
	{
		int nLast = m_nOTMax;
		while( nLast > 0 )
		{
			BOOL bFlag = FALSE;
			int nCnt = 1;
			// P̃[vōővfE[Ɉړ
			while( nCnt < nLast )
			{
				if( m_D3DObj[ m_nOT[ nCnt - 1 ] ].m_nPrio > m_D3DObj[ m_nOT[ nCnt - 0 ] ].m_nPrio )
				{
					// אڗvfւ
					int nOT = m_nOT[ nCnt - 1 ];
					m_nOT[ nCnt - 1 ] = m_nOT[ nCnt - 0 ];
					m_nOT[ nCnt - 0 ] = nOT;

					// ւ
					bFlag = TRUE;
				}
				// 
				nCnt++;
			}
			// I
			if( !bFlag ) break;
			// ővf炷
			nLast--;
		}
	}
	return TRUE;
}

/*******************************************************************************
 *	@fn				int CDLGraph::Update( void )
 *	@retval			int							TRUE=OK/FALSE=NG
 *	@brief			ʍXV
 ******************************************************************************/
int CDLGraph::Update( void )
{
	HRESULT hr = D3D_OK;

	// V[̃NA
	hr = m_lpD3DDev->Clear( 0,
							NULL,
							D3DCLEAR_TARGET,
							D3DCOLOR_ARGB( 0x00, 0x00, 0x00, 0x00 ),
							1.0f,
							0 );
	if( FAILED( hr ) )
	{
		DXTRACE_ERR( "Clear()", hr );
		return FALSE;
	}

	// V[̕`Jn
	hr = m_lpD3DDev->BeginScene();
	if( SUCCEEDED( hr ) )
	{
		// V[̕`
		for( int i = 0; i < OBJECT_MAX; i++ )
		{
			// IuWFNg`攻
			if( i >= m_nOTMax ) break;

			D3DOBJECT *pObj = &m_D3DObj[ m_nOT[ i ] ];

			// gpĂȂꍇ͕`悵Ȃ
			if( !pObj->m_bUsed ) continue;
			// `悪LɂȂĂȂꍇ͕`悵Ȃ
			if( pObj->m_eDraw != enumDraw_On ) continue;

			// IuWFNg`
			( this->*StepDraw[ pObj->m_eObj ] )( pObj );
		}

		// V[̕`I
		m_lpD3DDev->EndScene();
	}

	// V[̕\
	hr = m_lpD3DDev->Present( NULL, NULL, NULL, NULL );
	// foCX̕
	if( !DeviceRestore( hr ) )
	{
		DXTRACE_ERR( "DeviceRestore()", hr );
		return FALSE;
	}

	return TRUE;
}

/*******************************************************************************
 *	@fn				int CDLGraph::ChangeDisplay( const SDLParam& SDL )
 *	@param[in]		SDL							DL\
 *	@retval			int							TRUE=OK/FALSE=NG
 *	@brief			ʐ؂ւ
 ******************************************************************************/
int CDLGraph::ChangeDisplay( const SDLParam& SDL )
{
	HRESULT hr = D3D_OK;

	// \TCY̐ݒ
	m_D3DPP.BackBufferWidth		= SDL.m_nWidth;
	m_D3DPP.BackBufferHeight	= SDL.m_nHeight;
	// rfI[hݒ
	m_D3DPP.Windowed			= SDL.m_bMode;

	// ʂ̐؂ւ
	hr = m_lpD3DDev->Reset( &m_D3DPP );
	if( !DeviceRestore( hr ) )
	{
		DXTRACE_ERR( "DeviceRestore()", hr );
		return FALSE;
	}

	// r[|[g̐ݒ
	if( !SetViewPort( SDL ) )
		return FALSE;

	return TRUE;
}

/*******************************************************************************
 *	@fn				int CDLGraph::Load( const SDLMaterial& SDL )
 *	@param[in]		SDL							fޏ\
 *	@retval			int							fޔԍ
 *	@brief			fޓǂݍ
 ******************************************************************************/
int CDLGraph::Load( const SDLMaterial& SDL )
{
	_ASSERT( SDL.m_nID < TEXTURE_MAX );
	// fޓǂݍ
	if( !( this->*StepLoad[ SDL.m_eMat - enumMaterial_GraphStart ] )( SDL, SDL.m_nID ) )
		return -1;
	return SDL.m_nID;
}

/*******************************************************************************
 *	@fn				int CDLGraph::Remove( const SDLMaterial& SDL )
 *	@param[in]		SDL							fޏ\
 *	@retval			int							TRUE=OK/FALSE=NG
 *	@brief			fމ
 ******************************************************************************/
int CDLGraph::Remove( const SDLMaterial& SDL )
{
	_ASSERT( SDL.m_nID < TEXTURE_MAX );
	// fމ
	if( !( this->*StepRemove[ SDL.m_eMat - enumMaterial_GraphStart ] )( SDL.m_nID ) )
		return FALSE;
	return TRUE;
}

/*******************************************************************************
 *	@fn				int CDLGraph::Add( const SDLGraph& SDL )
 *	@param[in]		SDL							Graphics\
 *	@retval			int							IuWFNgԍ
 *	@brief			IuWFNgǉ
 ******************************************************************************/
int CDLGraph::Add( const SDLGraph& SDL )
{
	_ASSERT( SDL.m_nID < OBJECT_MAX );
	// IuWFNgǉ
	if( !( this->*StepAdd[ SDL.m_eObj ] )( SDL, SDL.m_nID ) )
		return -1;
	return SDL.m_nID;
}

/*******************************************************************************
 *	@fn				int CDLGraph::Del( const SDLGraph& SDL )
 *	@param[in]		SDL							Graphics\
 *	@retval			int							TRUE=OK/FALSE=NG
 *	@brief			IuWFNg폜
 ******************************************************************************/
int CDLGraph::Del( const SDLGraph& SDL )
{
	_ASSERT( SDL.m_nID < OBJECT_MAX );
	// IuWFNg폜
	if( !( this->*StepDel[ SDL.m_eObj ] )( SDL.m_nID ) )
		return FALSE;
	return TRUE;
}

}	// namespace	DL
